#!perl -w
#
# FsTab -- encapsulate /etc/fstab.
#   Copyright (C) 2005  Erik van Konijnenburg
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
use strict;
use warnings;
use Base;
use Conf;
use FsOpts;
use FsEntry;
package FsTab;


my $fsTab = undef;


sub init () {
	if (defined ($fsTab)) {
		return;
	}
	$fsTab = [];
	my $name = Conf::get('fstab');
	if (! open (IN, "<", "$name")) {
		Base::fatal ("can't read $name");
	}
	while (defined (my $line = <IN>)) {
		$line =~ s/^\s+//;
		next if $line =~ /^#/;
		my @fields = split (/\s+/, $line, 999);
		if ($#fields < 6) {
			# no test for extra fields;
			# the mount command allows that.
			Base::fatal ("malformed line in $name");
		}
		if ($fields[2] eq "swap") {
			if (($fields[1] ne "none") &&($fields[1] ne "swap")) {
				Base::fatal ("bad swap entry in $name");
			}
		}
		
		my $opts = FsOpts->new (
			string => unmangle($fields[3]));

		my $descr = FsEntry->new(
			dev => unmangle($fields[0]),
			mnt => unmangle($fields[1]),
			type => unmangle($fields[2]),
			opts => $opts,
			);
		push @{$fsTab}, $descr;
	}
	if (! close (IN)) {
		Base::fatal ("could not read $name");
	}
}

sub all	() {
	init;
	return $fsTab;
}

sub findByMountPoint ($) {
	my ($mnt) = @_;
	my $result;
	for my $entry (@{FsTab::all()}) {
		if ($entry->mnt() eq $mnt) {
			if (defined ($result)) {
				Base::fatal ("duplicate mount point in fstab $mnt");
			}
			$result = $entry;
		}
	}
	if (! defined ($result)) {
		Base::fatal ("mount point not in fstab: $mnt");
	}
	return $result;
}

# replace octal escapes
sub unmangle ($) {
	my ($string) = @_;
	while ($string =~ /\\([0-7]{3})/) {
		$string = $` . chr (oct ($1)) . $';
	}
	return $string;
}

1;
