#!perl -w
#
# ModDir -- inventory of modules in /lib/modules/2.6.x
#   Copyright (C) 2005  Erik van Konijnenburg
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
use strict;
use warnings;
use Base;
use Conf;
package ModDir;

my $modTab = undef;
my $modList = undef;

sub walkModules ($);

sub init () {
	if (defined ($modTab)) {
		return;
	}
	$modTab = {};
	my $modDir = Conf::get('modDir');
	walkModules ($modDir);
	$modList = [ values %{$modTab} ];
}

sub walkModules ($) {
	my ($dirName) = @_;
	my $dir;
	my $modExt = Conf::get('modExt');
	if (! opendir ($dir, $dirName)) {
		Base::fatal ("can't open directory $dirName");
	}
	while (defined(my $entry = readdir($dir))) {
		next if $entry eq ".";
		next if $entry eq "..";
		my $path = "$dirName/$entry";
		if (-l $path) {
			next;
		}
		elsif (-f _) {
			if ($entry =~ /^(.*)\.$modExt$/) {
				my $module = $1;
				$module =~ s!_!-!g;
				if (exists ($modTab->{$module})) {
					Base::fatal ("duplicate module $module");
				}
				$modTab->{$module} = $path;
			}
		}
		elsif (-d _) {
			walkModules ($path);
		}
	}
	if (! closedir ($dir)) {
		Base::fatal ("could not read directory $dirName");
	}
}

#
# findByModnam -- return path to module object given module name.
#
sub findByModnam ($) {
	my ($module) = @_;
	init;
	$module =~ s!_!-!g;
	return $modTab->{$module};
}

sub all	() {
	init;
	return $modList;
}

1;
